
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 2D Subsampling:

TRUE = (1==1);
FALSE = ~TRUE;

TEST_SCRIPT = FALSE;  %% Set to true for testing path ok.

% Everything we've learned about 1D subsampling applies in 2D as
% well.  We can perfectly reconstruct a subsampled image as long
% as the image is appropriately (according to the Nyquist sampling
% theorem) bandlimited.

% Here's a simple example:
%

im0 = pgmRead('einCrop.pgm');
im = im0;

figure(1), clf;
showIm(im);
title('Face image')
minAbs = 1.0e-1;
figure(2), clf;
showIm(log10(fftshift(abs(fft2(im))) + minAbs));
title('DFT of face image');

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subSamp = 3;
impulses = zeros(size(im)); 
impulses(1:subSamp:size(im,1),1:subSamp:size(im,2)) = 1;
figure(1); clf;
showIm(impulses,'auto2');
title('Impulses')
figure(2); clf;
showIm(fftshift(abs(fft2(impulses))));
title('DFT of impulses')

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

im_impulses = im .*  impulses;
figure(1); clf;
showIm(im_impulses);
title('Face image times impulses'), drawnow
dft_im_impulses = fftshift(fft2(im_impulses));
figure(2); clf;
showIm(log10(abs(dft_im_impulses)+minAbs));
title('DFT of Face image times impulses'), drawnow

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We get replicas of the original Fourier transform when we
% multiply by an impulse train in the spatial domain.  We can
% reconstruct the original image by pulling out the correct
% replica:
%
ix0 = floor(size(im,2)/2) + 1;
iy0 = floor(size(im,1)/2) + 1;
repX = ceil(size(im,2)/subSamp);
repY = ceil(size(im,1)/subSamp);
box = zeros(size(im));
p0 = [ix0 - floor(repX/2), iy0 - floor(repY/2)];
box(p0(2)+[0:repY], p0(1) + [0:repX]) =  1;
box(p0(2)+[0:repY], p0(1)) = 0.5;
box(p0(2)+[0:repY], p0(1)+repX) = 0.5;
box(p0(2), p0(1)+[0:repX]) = 0.5;
box(p0(2)+repY, p0(1)+[0:repX]) = 0.5;
box(p0(2)+[0, repY], p0(1) + [0, repX]) =  0.25;

%box = fftshift(box);
reconstructed_im = subSamp^2 * real(ifft2(ifftshift(box.*dft_im_impulses)));
figure(2); clf;
showIm(log10(abs(box.*dft_im_impulses)+minAbs));
title('Low-Pass of DFT of Face image times impulses'), drawnow

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1), clf
showIm(reconstructed_im);
title('Reconstructed Image');
figure(2); clf;
showIm(im-reconstructed_im, [-100 100]);
title('Error Image');
imStats(im,reconstructed_im)

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(2); clf;
showIm(im(1:subSamp:end, 1:subSamp:end) - ...
       reconstructed_im(1:subSamp:end, 1:subSamp:end), [-0.1 0.1]);
title('Reconstruction Error on Subsampled Grid');
imStats(im(1:subSamp:end, 1:subSamp:end) - ...
        reconstructed_im(1:subSamp:end, 1:subSamp:end));

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Filtering and Subsampling.

% When we use a pre-filter to bandlimit an image, the filtered
% image can be subsampled without aliasing.

% First look at the dft of an image that is not sufficiently band
% limited: 
%
figure(1), clf;
showIm(im);
title('Face image')

dft_im = fft2( im);
figure(2), clf;
showIm(log10(fftshift(abs(dft_im))+minAbs));
title('Energy everywhere')


if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Now blur the image to produce a (more or less) band limited
% result:
%
blur_im = cconv2sep(im0,binomialFilter(9),binomialFilter(9));
figure(1), clf;
showIm(blur_im);
title('Face blur_image')

dft_blur_im = fft2(blur_im);
figure(2); clf;
showIm(log10(fftshift(abs(dft_blur_im))+minAbs));
title('No energy at high freqs')

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subSamp = 3;
impulses = zeros(size(blur_im)); 
impulses(1:subSamp:size(blur_im,1),1:subSamp:size(blur_im,2)) = 1;
figure(1); clf;
showIm(impulses,'auto2');
title('Impulses')
figure(2); clf;
showIm(fftshift(abs(fft2(impulses))));
title('DFT of impulses')

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

blur_im_impulses = blur_im .*  impulses;
figure(1); clf;
showIm(blur_im_impulses);
title('Face blur_image times impulses');
dft_blur_im_impulses = fftshift(fft2(blur_im_impulses));
figure(2); clf;
showIm(log10(abs(dft_blur_im_impulses)+minAbs));
title('DFT of Face blur_image times impulses');

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We get replicas of the original Fourier transform when we
% multiply by an impulse train in the spatial domain.  We can
% reconstruct the original image by pulling out the correct
% replica:
%
ix0 = floor(size(im,2)/2) + 1;
iy0 = floor(size(im,1)/2) + 1;
repX = ceil(size(im,2)/subSamp);
repY = ceil(size(im,1)/subSamp);
box = zeros(size(im));
p0 = [ix0 - floor(repX/2), iy0 - floor(repY/2)];
box(p0(2)+[0:repY], p0(1) + [0:repX]) =  1;
box(p0(2)+[0:repY], p0(1)) = 0.5;
box(p0(2)+[0:repY], p0(1)+repX) = 0.5;
box(p0(2), p0(1)+[0:repX]) = 0.5;
box(p0(2)+repY, p0(1)+[0:repX]) = 0.5;
box(p0(2)+[0, repY], p0(1) + [0, repX]) =  0.25;

%box = fftshift(box);
reconstructed_blur_im = subSamp^2 * real(ifft2(ifftshift(box.*dft_blur_im_impulses)));
figure(2); clf;
showIm(log10(abs(box.*dft_im_impulses)+minAbs));
title('DFT of Face blur_image times impulses');
figure(1), clf
showIm(reconstructed_blur_im);
title('Reconstructed Image');

if TEST_SCRIPT
  pause(1);
else
  fprintf(1, '\nHit any key to continue...');
  pause;
  fprintf(1, '\n');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


figure(2); clf;
showIm(blur_im-reconstructed_blur_im, [-100 100]);
title('Error Image');
imStats(blur_im,reconstructed_blur_im)

